import logging
import os
from typing import Optional
from git import Repo
from github import Github, Auth 
from github.GitRelease import GitRelease
from releaser.utilities import file_util
from . import errors_util, helpers

_logger:logging.Logger = logging.getLogger(__name__)

class GitHubRepository() :
    """
    Utility class for interacting with GitHub.
    """

    def __init__(self, git_repository:Repo) :
        helpers.assertSet(_logger, "GitHub::The git repository is not set", git_repository)
        self._git_repository:Repo = git_repository
       
        self._token:str = self._getGitHubToken()
        self._github:Github = Github(auth=Auth.Token(self._token))
        
        
    def createRelease(self, release_name:str, release_description:str, tagName:str) -> GitRelease:
        """
        Create a new release on GitHub.

        Args:
            release_name (str): The name of the release.
            release_description (str): The description of the release.
            tagName (str): The name of the tag to create the release from.

        Returns:
            GitRelease: The release.
        """
        helpers.assertSet(_logger, "GitHub::The release name is not set", release_name)
        helpers.assertSet(_logger, "GitHub::The release description is not set", release_description)
        helpers.assertSet(_logger, "GitHub::The tag name is not set", tagName)
        return self._getGitHubRepository().create_git_release(tagName, name=release_name, message=release_description, draft=False, prerelease=False)
 
 
    def uploadFileToRelease(self, release:GitRelease, file_name:str, file_path:str, content_type:str = "") :
        """
        Upload a file to a release on GitHub.

        Args:
            release (GitRelease): The release to upload the file to.
            file_name (str): The name of the file to upload.
            file_path (str): The path to the file to upload.
            content_type (str): The content type of the file to upload.
            
        Raises:
            GitHubError: If the file does not exist or is not a file.
        """
        helpers.assertSet(_logger, "GitHub::The release is not set", release)
        helpers.assertSet(_logger, "GitHub::The file name is not set", file_name)
        helpers.assertSet(_logger, "GitHub::The file path is not set", file_path)
        
        _logger.info(f"Uploading {file_path} to release.")
        
        # Check the file exists and is a file
        if file_util.exists(file_path) and file_util.isFile(file_path) : 
            if(helpers.hasValue(content_type)) :
                release.upload_asset(file_path, content_type=content_type, name=file_name)
            else :
                release.upload_asset(file_path, name=file_name)
        else :
            raise GitHubError(f"Cannot upload file to release ({release.name}) - the file does not exist ({file_path}).")
            
        _logger.info(f"Uploaded {file_path} to release.")


    def _getGitHubRepository(self):
        """
        Get the GitHub repository.

        Returns:
            Github: The GitHub repository.
        """
        return self._github.get_repo(self.getRepositoryName())
           
            
    def _getGitHubToken(self) -> str:
        """
        Get the GitHub token from the environment variable GITHUB_TOKEN.

        Raises:
            errors_util.ProjectError: If the GITHUB_TOKEN environment variable is not set.

        Returns:
            str: The token.
        """
        token:Optional[str] = os.getenv('GITHUB_TOKEN')
        if not token:
            raise GitHubError("GITHUB_TOKEN environment variable is not set")
        return token


    def getRepositoryName(self) -> str:
        """
        Get the name (Owner/Repository) of this repository.
        
        Returns:
            str: The name (Owner/Repository) of this repository.
        """
        return self._determineRepositoryName()
    
    
    def _determineRepositoryName(self) -> str:
    
        url:str = self._getRepository().remotes.origin.url
                
        # Remove protocol and domain
        if url.startswith("https://github.com/"):
            repo_path = url[len("https://github.com/"):]
        elif url.startswith("git@github.com:"):
            repo_path = url[len("git@github.com:"):]
        else:
            raise GitHubError(f"The repository is a GitHub repository ({url}).")

        # Remove .git suffix if present
        if repo_path.endswith(".git"):
            repo_path = repo_path[:-4]

        # Remove trailing slash if present
        return repo_path.rstrip("/")


    def _getRepository(self) -> Repo:
        """
        Get the repository from the git repository.

        Returns:
            Repo: The repository.
        """
        return self._git_repository


class GitHubError(errors_util.UtilityError) :
    """Raised by the this utility function to indicate some issue."""



import logging
from datetime import datetime, timedelta

_logger:logging.Logger = logging.getLogger(__name__)

def getCurrentDateTimeString(date_format:str = "%d/%m/%Y %H:%M:%S") :
    """
    Returns the current date and time as a formatted string.

    Args:
        date_format (str, optional): The format string to use for formatting the date and time. Defaults to "%d/%m/%Y %H:%M:%S".

    Returns:
        str: A string representing the current date and time formatted according to the specified format.
    """
    return datetime.now().strftime(date_format)


def howOld(timestamp:float) -> timedelta:
    """
    Returns the time delta from the given timestamp to now.

    Args:
        timestamp (float): The timestamp to compare against the current time.

    Returns:
        timedelta: A timedelta object representing the difference between the current time and the given timestamp.
    """
    return datetime.today() - datetime.fromtimestamp(timestamp)


def howOldDays(timestamp:int) -> int:
    """
    Returns the number of days old the given timestamp is.

    Args:
        timestamp (int): The timestamp to compare against the current time.

    Returns:
        int: The number of days old the given timestamp is.
    """
    return howOld(timestamp).days


class ProjectError(Exception):
    """
    Base class for all raised by this project.
    Using this as a base class for all custom errors allows developers to use except ProjectException to trap these project-based custom exceptions.
    """

    
class UtilityError(ProjectError) :
    """Raised by utility functions.""" 



import logging
import logging.handlers
import sys
import os.path

# Logging
_logger = logging.getLogger(__name__)  # module name

def setupRootLogging(logToFile:str):
    """
    Sets up the root logger to log to both stdout and a file.
    This function creates a directory for the log file if it does not exist,
    and configures the logger to write debug-level messages to the file and info-level messages to stdout.

    Args:
        logToFile (str): The path to the log file where debug messages will be written.
    """
    root = logging.getLogger(None)
    root.setLevel(logging.DEBUG)

    formatter = logging.Formatter('%(asctime)s : %(levelname)s : %(module)s.%(funcName)s(%(lineno)d) >> %(message)s')

    stdout_handler = logging.StreamHandler(sys.stdout)
    stdout_handler.setLevel(logging.INFO)
    stdout_handler.setFormatter(formatter)
    root.addHandler(stdout_handler)

    print(f"Setting up root logging to {logToFile}")    
    os.makedirs(os.path.dirname(logToFile), 0o755, True)  # make sure the parent directory exists
        
    file_handler = logging.handlers.RotatingFileHandler(logToFile, "a", 10*1024*1024, 3)
    file_handler.setLevel(logging.DEBUG)
    file_handler.setFormatter(formatter)
    root.addHandler(file_handler)

    _logger.debug("Logging set up")



#!/usr/bin/env python3

import argparse
import logging
import traceback
import dotenv

from releaser.utilities import github_util, helpers, log_util, git_util, file_util, zip_util, errors_util, time_util
import releaser.constants as constants

# Logging
_logger:logging.Logger = logging.getLogger(__name__)


# Sets up the whole shebang
def _init() :
    dotenv.load_dotenv()
    log_util.setupRootLogging(constants.LOG_TO_FILE)


# Deals with all the command-line interface
def _commandRunner() :
    parser = argparse.ArgumentParser(description="Fetch and resolve external dependencies for a project.", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    subparsers = parser.add_subparsers()
    _buildFrontend(subparsers)
    _buildAndReleaseFrontend(subparsers)
    _buildBackend(subparsers)
    _buildAndReleaseBackend(subparsers)
    _buildRepository(subparsers)
    _buildAndRelease(subparsers)

    args:argparse.Namespace = parser.parse_args()
    args.func(args)


# Builds the frontend release.
def _buildFrontend(subparsers) :
    runner = subparsers.add_parser("build-frontend", help="Builds the frontend release.", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    runner.add_argument("--repo", "-r", help='URL to the the repo to clone', default=constants.FRONTEND_REPO_URL)
    runner.add_argument("--branch", "-b", help='The branch to use', default=constants.FRONTEND_REPO_BRANCH)
    runner.add_argument("--repo_target_dir", "-c", help='Where to clone the repo to (warning: existing directories will be emptied first)', default=constants.FRONTEND_CLONE_DIR)
    runner.add_argument("--release_target_dir", "-t", help='Where to put the zipped release', default=constants.RELEASE_DIR)
    runner.add_argument("--release_file_name", "-f", help='The name to use.', default=constants.FRONTEND_RELEASE_NAME) 
    runner.add_argument("--clean_patterns", "-p", help='A path to a file containing a list of files to be removed from the repository prior to creating the release.', default=constants.CLEAN_PATTERNS_FILE)
    runner.set_defaults(func=_buildCommand)


# Builds and releases the frontend.
def _buildAndReleaseFrontend(subparsers) :
    runner = subparsers.add_parser("release-frontend", help="Builds and releases the frontend release.", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    runner.add_argument("--repo", "-r", help='URL to the the repo to clone', default=constants.FRONTEND_REPO_URL)
    runner.add_argument("--branch", "-b", help='The branch to use', default=constants.FRONTEND_REPO_BRANCH)
    runner.add_argument("--repo_target_dir", "-c", help='Where to clone the repo to (warning: existing directories will be emptied first)', default=constants.FRONTEND_CLONE_DIR)
    runner.add_argument("--release_target_dir", "-t", help='Where to put the zipped release', default=constants.RELEASE_DIR)
    runner.add_argument("--release_file_name", "-f", help='The name to use.', default=constants.FRONTEND_RELEASE_NAME) 
    runner.add_argument("--clean_patterns", "-p", help='A path to a file containing a list of files to be removed from the repository prior to creating the release.', default=constants.CLEAN_PATTERNS_FILE)
    runner.add_argument("--tag_version", help='The name of the tag to create. E.g. v1.0.1. Cannot be the same as a previous tag version.', required=True)
    runner.add_argument("--tag_description", help='The description of the tag to create.', required=True)
    runner.add_argument("--release_version", help='The name of the release to create E.g. v1.11.0. Cannot be the same as a previous release version. Defaults to the tag version.', required=False)
    runner.add_argument("--release_description", help='The description of the release to create Defaults to the tag version.', required=False)
    runner.set_defaults(func=_buildAndReleaseCommand)


# Builds the backend release.
def _buildBackend(subparsers) :
    runner = subparsers.add_parser("build-backend", help="Builds the backend release.", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    runner.add_argument("--repo", "-r", help='URL to the the repo to clone', default=constants.BACKEND_REPO_URL)
    runner.add_argument("--branch", "-b", help='The branch to use', default=constants.BACKEND_REPO_BRANCH)
    runner.add_argument("--repo_target_dir", "-c", help='Where to clone the repo to (warning: existing directories will be emptied first)', default=constants.BACKEND_CLONE_DIR)
    runner.add_argument("--release_target_dir", "-t", help='Where to put the zipped release', default=constants.RELEASE_DIR)
    runner.add_argument("--release_file_name", "-f", help='The name to use.', default=constants.FRONTEND_RELEASE_NAME) 
    runner.add_argument("--clean_patterns", "-p", help='A path to a file containing a list of files to be removed from the repository prior to creating the release.', default=constants.CLEAN_PATTERNS_FILE)
    runner.set_defaults(func=_buildCommand)


# Builds and releases the backend.
def _buildAndReleaseBackend(subparsers) :
    runner = subparsers.add_parser("release-backend", help="Builds and releases the backend release.", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    runner.add_argument("--repo", "-r", help='URL to the the repo to clone', default=constants.BACKEND_REPO_URL)
    runner.add_argument("--branch", "-b", help='The branch to use', default=constants.BACKEND_REPO_BRANCH)
    runner.add_argument("--repo_target_dir", "-c", help='Where to clone the repo to (warning: existing directories will be emptied first)', default=constants.BACKEND_CLONE_DIR)
    runner.add_argument("--release_target_dir", "-t", help='Where to put the zipped release', default=constants.RELEASE_DIR)
    runner.add_argument("--release_file_name", "-f", help='The name to use.', default=constants.FRONTEND_RELEASE_NAME)  
    runner.add_argument("--clean_patterns", "-p", help='A path to a file containing a list of files to be removed from the repository prior to creating the release.', default=constants.CLEAN_PATTERNS_FILE)
    runner.add_argument("--tag_version", help='The name of the tag to create. E.g. v1.0.1. Cannot be the same as a previous tag version.', required=True)
    runner.add_argument("--tag_description", help='The description of the tag to create.', required=True)
    runner.add_argument("--release_version", help='The name of the release to create E.g. v1.11.0. Cannot be the same as a previous release version. Defaults to the tag version.', required=False)
    runner.add_argument("--release_description", help='The description of the release to create Defaults to the tag version.', required=False)
    runner.set_defaults(func=_buildAndReleaseCommand)


# Builds a release for the specified repository.
def _buildRepository(subparsers) :
    runner = subparsers.add_parser("build", help="Builds a release for the specified repository.", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    runner.add_argument("--repo", "-r", help='URL to the the repo to clone')
    runner.add_argument("--branch", "-b", help='The branch to use', default="main")
    runner.add_argument("--repo_target_dir", "-c", help='Where to clone the repo to (warning: existing directories will be emptied first)', default=constants.CLONE_DIR)
    runner.add_argument("--release_target_dir", "-t", help='Where to put the zipped release', default=constants.RELEASE_DIR)
    runner.add_argument("--release_file_name", "-f", help='The name to use.', default=f"archive-{time_util.getCurrentDateTimeString(date_format='%Y%m%d')}.zip")
    runner.add_argument("--clean_patterns", "-p", help='A path to a file containing a list of files to be removed from the repository prior to creating the release.', default=constants.CLEAN_PATTERNS_FILE)
    runner.set_defaults(func=_buildCommand)


# Builds and releases the backend.
def _buildAndRelease(subparsers) :
    runner = subparsers.add_parser("release", help="Build and release the specified repository.", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    runner.add_argument("--repo", "-r", help='URL to the the repo to clone')
    runner.add_argument("--branch", "-b", help='The branch to use', default="main")
    runner.add_argument("--repo_target_dir", "-c", help='Where to clone the repo to (warning: existing directories will be emptied first)', default=constants.CLONE_DIR)
    runner.add_argument("--release_target_dir", "-t", help='Where to put the zipped release', default=constants.RELEASE_DIR)
    runner.add_argument("--release_file_name", "-f", help='The name to use.', default=f"archive-{time_util.getCurrentDateTimeString(date_format='%Y%m%d')}.zip")
    runner.add_argument("--clean_patterns", "-p", help='A path to a file containing a list of files to be removed from the repository prior to creating the release.', default=constants.CLEAN_PATTERNS_FILE)
    runner.add_argument("--tag_version", help='The name of the tag to create. E.g. v1.0.1. Cannot be the same as a previous tag version.', required=True)
    runner.add_argument("--tag_description", help='The description of the tag to create.', required=True)
    runner.add_argument("--release_version", help='The name of the release to create E.g. v1.11.0. Cannot be the same as a previous release version. Defaults to the tag version.', required=False)
    runner.add_argument("--release_description", help='The description of the release to create Defaults to the tag version.', required=False)
    runner.set_defaults(func=_buildAndReleaseCommand)


def _buildCommand(args:argparse.Namespace) :
    """
    Builds the release from the given repository and branch to the given directory and name.

    Args:
        args (argparse.Namespace): The arguments passed to the command.
    """
    _build(args.repo, args.branch, args.repo_target_dir, args.clean_patterns, args.release_target_dir, args.release_file_name)


def _build(repository_url:str, repository_branch:str, repository_target_dir:str, patterns_file:str, release_target_dir:str, release_target_file_name:str):
    """
    Builds the release from the given repository and branch to the given directory and name.

    Args:
        repository_url (str): The Url of the repository to create the release for.
        repository_branch (str): The branch of the repository to use.
        repository_target_dir (str): The directory to clone the repository to.
        patterns_file (str): Path to a file containing patterns of files to remove before creating the release.
        release_target_dir (str): The directory to place the release in.
        release_target_file_name (str): The name of the release file.
    """
    helpers.assertSet(_logger, "_build::repository_url not set", repository_url)
    _validateRepositoryUrl(repository_url)
    helpers.assertSet(_logger, "_build::repository_branch not set", repository_branch)
    helpers.assertSet(_logger, "_build::repository_target_dir not set", repository_target_dir)
    helpers.assertSet(_logger, "_build::patterns_file not set", patterns_file)
    helpers.assertSet(_logger, "_build::release_target_dir not set", release_target_dir)
    helpers.assertSet(_logger, "_build::release_target_file_name not set", release_target_file_name)

    _logger.info(f"Building {release_target_file_name} for {repository_url}:{repository_branch}")

    # Clone the repository from the given path
    _cloneRepository(repository_url=repository_url, repository_branch=repository_branch, repository_target_dir=repository_target_dir)

    # Build the release
    _buildRelease(repository_target_dir=repository_target_dir, patterns_file=patterns_file, release_target_dir=release_target_dir, release_target_name=release_target_file_name)

    _logger.info(f"{release_target_file_name} built successfully.")


def _buildAndReleaseCommand(args:argparse.Namespace) :
    """
    Builds and releases the frontend release.

    Args:
        args (argparse.Namespace): The arguments passed to the command.
    """
    # If the release version or description is not set, use the tag's information
    release_version:str = args.release_version if helpers.hasValue(args.release_version) else args.tag_version
    release_description:str = args.release_description if helpers.hasValue(args.release_description) else args.tag_description

    _buildAndReleaseToGitHub(args.repo, args.branch, args.repo_target_dir, args.clean_patterns, args.release_target_dir, args.release_file_name, args.tag_version, args.tag_description, release_version, release_description)


def _buildAndReleaseToGitHub(repository_url:str, repository_branch:str, repository_target_dir:str, patterns_file:str, release_target_dir:str, release_target_file_name:str, tag_version:str, tag_description:str, release_version:str, release_description:str) :
    """
    Builds the release from the given repository and branch to the given directory and name.

    Args:
        repository_url (str): The Url of the repository to create the release for.
        repository_branch (str): The branch of the repository to use.
        repository_target_dir (str): The directory to clone the repository to.
        patterns_file (str): Path to a file containing patterns of files to remove before creating the release.
        release_target_dir (str): The directory to place the release in.
        release_target_file_name (str): The name of the release file.
        tag_version (str): The name of the tag to create.
        tag_description (str): The description of the tag to create.
        release_version (str): The name of the release to create.
        release_description (str): The description of the release to create.
    """
    helpers.assertSet(_logger, "_buildAndReleaseToGitHub::repository_url not set", repository_url)
    _validateRepositoryUrl(repository_url)
    helpers.assertSet(_logger, "_buildAndReleaseToGitHub::repository_branch not set", repository_branch)
    helpers.assertSet(_logger, "_buildAndReleaseToGitHub::repository_target_dir not set", repository_target_dir)
    helpers.assertSet(_logger, "_buildAndReleaseToGitHub::patterns_file not set", patterns_file)
    helpers.assertSet(_logger, "_buildAndReleaseToGitHub::release_target_dir not set", release_target_dir)
    helpers.assertSet(_logger, "_buildAndReleaseToGitHub::release_target_file_name not set", release_target_file_name)
    helpers.assertSet(_logger, "_buildAndReleaseToGitHub::tag_version not set", tag_version)
    helpers.assertSet(_logger, "_buildAndReleaseToGitHub::tag_description not set", tag_description)
    helpers.assertSet(_logger, "_buildAndReleaseToGitHub::release_version not set", release_version)
    helpers.assertSet(_logger, "_buildAndReleaseToGitHub::release_description not set", release_description)

    _logger.info(f"Building release for {repository_url}:{repository_branch}")
    
    # Clone the repository from the given path
    repository:git_util.GitRepository = _cloneRepository(repository_url=repository_url, repository_branch=repository_branch, repository_target_dir=repository_target_dir)

    # Create the tag
    _createTag(repository=repository, tag_name=tag_version, tag_description=tag_description)

    # Create the release - the build cleans the repository, potentially including the .git directory, so create the release while we still can
    github:github_util.GitHubRepository = github_util.GitHubRepository(repository.getRepository())
    release:github_util.GitRelease = github.createRelease(release_name=release_version, release_description=release_description, tagName=tag_version)

    # Build the release
    release_path:str = _buildRelease(repository_target_dir=repository_target_dir, patterns_file=patterns_file, release_target_dir=release_target_dir, release_target_name=release_target_file_name)

    # Upload the release build to the release
    github.uploadFileToRelease(release=release, file_name=release_target_file_name, file_path=release_path, content_type="application/zip")

    _logger.info("Release build completed successfully.")


def _validateRepositoryUrl(repository_url:str):
    """
    Validates the repository URL. Exits if the URL is not valid

    Args:
        repository_url (str): The URL of the repository to validate.
    """
    if not repository_url or not isinstance(repository_url, str) or not helpers.isValidUrl(repository_url) :
        _logger.error(f"Invalid repository URL provided: {repository_url}")
        return exit(1)


def _cloneRepository(repository_url:str, repository_branch:str, repository_target_dir:str) -> git_util.GitRepository :
    """
    Clones the repository from the given path and initializes any submodules.

    Args:
        repository_url (str): The Url of the repository to clone.
        repository_branch (str): The branch of the repository to use.
        repository_target_dir (str): The directory to clone the repository to.

    Returns:
        git_util.GitRepository: The cloned repository.

    Raises:
        git_util.GitError: If the repository cannot be cloned.
    """
    _logger.info(f"Cloning repository from {repository_url}:{repository_branch} to {repository_target_dir}...")

    # Prepare the repository target directory
    _prepareRepositoryTargetDirectory(repository_target_dir)

    # Clone the repository from the given path
    repository:git_util.GitRepository = git_util.GitRepository.cloneRepositoryBranch(repo_url=repository_url, branch=repository_branch, clone_target_dir=repository_target_dir)

    # Initialize any submodules in the repository
    repository.initAnySubmodules()

    _logger.info(f"...cloned repository from {repository_url}:{repository_branch} to {repository_target_dir}")

    # Return the repository
    return repository


def _prepareRepositoryTargetDirectory(repository_target_dir:str) :
    """
    Prepares the repository target directory by creating it if it doesn't exist and deleting its contents if it does.

    Args:
        repository_target_dir (str): The directory to prepare.

    Raises:
        errors_util.ProjectError: If the directory is not actually directory.
    """
    helpers.assertSet(_logger, "_prepareRepositoryTargetDirectory::repository_target_dir not set", repository_target_dir)
    _logger.info(f"Preparing repository target directory: {repository_target_dir}")

    if file_util.exists(repository_target_dir) :
        if not file_util.isDir(repository_target_dir) : 
            raise errors_util.ProjectError(f"{repository_target_dir} is not a directory.")
        _logger.info(f"Deleting contents of {repository_target_dir}")
        file_util.deleteContents(repository_target_dir)
    else :
        _logger.info(f"Creating directory: {repository_target_dir}")
        file_util.mkdir(repository_target_dir)


def _buildRelease(repository_target_dir:str, patterns_file:str, release_target_dir:str, release_target_name:str) -> str :
    """
    Builds the release from the given repository to the given directory and name.
    Simply cleans the repository of unwanted files and zips it up.
    Useful for repositories which are essentially a number of scripts rather than a single buildable application.

    Args:
        repository_target_dir (str): The directory to clone the repository to.
        patterns_file (str): Path to a file containing patterns of files to remove before creating the release.
        release_target_dir (str): The directory to place the release in.
        release_target_name (str): The name of the release file.
        
    Returns:
        str: The path to the zip file.
    """
    _logger.info(f"Building release in {repository_target_dir} to {release_target_dir}/{release_target_name}...")

    # Prepare the release target directory
    _prepareReleaseTargetDirectory(release_target_dir)

    # Clean the repository
    _cleanRepository(repository_target_dir=repository_target_dir, patterns_file=patterns_file)
    
    # Zip the repository - this is where the actual build happens
    return _zipRepository(repository_target_dir=repository_target_dir, release_target_dir=release_target_dir, release_target_name=release_target_name)


def _prepareReleaseTargetDirectory(release_target_dir:str) :
    """
    Prepares the release target directory by creating it if it doesn't exist.

    Args:
        release_target_dir (str): The directory to prepare.

    Raises:
        errors_util.ProjectError: If the directory is not actually directory.
    """
    helpers.assertSet(_logger, "_prepareReleaseTargetDirectory::release_target_dir not set", release_target_dir)
    _logger.info(f"Preparing release target directory: {release_target_dir}")
    
    if file_util.exists(release_target_dir) :
        if not file_util.isDir(release_target_dir) : 
            raise errors_util.ProjectError(f"{release_target_dir} is not a directory.")
    else :
        _logger.info(f"Creating directory: {release_target_dir}")
        file_util.mkdir(release_target_dir)


def _cleanRepository(repository_target_dir:str, patterns_file:str) :
    """
    Cleans the repository by removing the files of the given types.

    Args:
        repository_target_dir (str): The directory to clean.
        patterns_file (str): The file containing the patterns of files to remove.
    """
    _logger.info(f"Cleaning repository in {repository_target_dir}...")
    file_util.removeFilesOfTypes(repository_target_dir, file_util.readListFromFile(patterns_file))
    _logger.info(f"...cleaned repository in {repository_target_dir}")


def _zipRepository(repository_target_dir:str, release_target_dir:str, release_target_name:str) -> str :
    """
    Zips the repository to the given directory and name.

    Args:
        repository_target_dir (str): The directory to zip.
        release_target_dir (str): The directory to place the zip file in.
        release_target_name (str): The name of the zip file.
        
    Returns:
        str: The path to the zip file.
    """
    _logger.info(f"Zipping repository in {repository_target_dir} to {release_target_dir}/{release_target_name}...")
    return zip_util.zip(repository_target_dir, release_target_dir, release_target_name)


def _createTag(repository:git_util.GitRepository, tag_name:str, tag_description:str) :
    """
    Creates a tag in the repository.

    Args:
        repository (git_util.GitRepository): The repository to create the tag in.
        tag_name (str): The name of the tag to create.
        tag_description (str): The description of the tag to create.
    """
    _logger.info(f"Creating tag {tag_name} in {repository.getRepository().working_dir}...")
    repository.createTag(tag_name=tag_name, tag_description=tag_description)
    _logger.info(f"...created tag {tag_name} in {repository.getRepository().working_dir}")


def main() :
    try :
        _init()
        _commandRunner()
    except Exception :
        _logger.error(f"Command caught an exception (may not be harmful): {traceback.format_exc()}")
        raise


# the entry point
if __name__ == "__main__" :
    main()


import os
from .utilities import file_util, time_util

# A repository is needed, but not defined.
UNDEFINED_REPOSITORY:str = "Repository undefined."

# Constants for the Resolver application
HOME_DIR:str = os.getenv("RELEASER_HOME", os.getcwd())

# Default runtime directory
RUNTIME_DIR:str = os.getenv("RELEASER_RUNTIME_DIR", f"{HOME_DIR}/archive-and-release-runtime")

# Logging configuration
LOG_DIR:str = os.getenv("RELEASER_LOG_DIR", RUNTIME_DIR)
LOG_TO_FILE:str =f"{LOG_DIR}/releaser.log"

# Repository URLs and branches for the releaser.
BACKEND_REPO_URL:str = os.getenv("RELEASER_BACKEND_REPO_URL", UNDEFINED_REPOSITORY)
BACKEND_REPO_BRANCH:str = os.getenv("RELEASER_BACKEND_REPO_BRANCH", "main")
FRONTEND_REPO_URL:str = os.getenv("RELEASER_FRONTEND_REPO_URL", UNDEFINED_REPOSITORY)
FRONTEND_REPO_BRANCH:str = os.getenv("RELEASER_FRONTEND_REPO_BRANCH", "main")

# Directory to clone the repository to
CLONE_DIR:str =os.getenv("RELEASER_CLONE_DIR", f"{RUNTIME_DIR}/clonedRepository")
FRONTEND_CLONE_DIR:str =os.getenv("RELEASER_FRONTEND_CLONE_DIR", f"{CLONE_DIR}/frontend")
BACKEND_CLONE_DIR:str =os.getenv("RELEASER_BACKEND_CLONE_DIR", f"{CLONE_DIR}/backend")


# Directory to build the release to
RELEASE_DIR:str =os.getenv("RELEASER_RELEASE_DIR", f"{RUNTIME_DIR}/release")

# Default release names
FRONTEND_RELEASE_NAME:str = os.getenv("RELEASER_FRONTEND_RELEASE_NAME", f"frontend-{time_util.getCurrentDateTimeString(date_format='%Y%m%d')}.zip")
BACKEND_RELEASE_NAME:str = os.getenv("RELEASER_BACKEND_RELEASE_NAME", f"backend-{time_util.getCurrentDateTimeString(date_format='%Y%m%d')}.zip")

# Pattern file - clean.txt is a sibling to constants.py
CLEAN_PATTERNS_FILE:str = os.getenv("RELEASER_CLEAN_PATTERNS_FILE", file_util.buildPath(file_util.getParentDirectory(__file__), "clean.txt"))
